/*
	This code was written to support the book, "ARM Assembly for Embedded Applications",
	by Daniel W. Lewis. Permission is granted to freely share this software provided
	that this notice is not removed. This software is intended to be used with a run-time
    library adapted by the author from the STM Cube Library for the 32F429IDISCOVERY 
    board and available for download from http://www.engr.scu.edu/~dlewis/book3.
*/

#include <stdio.h>
#include <stdint.h>
#include "library.h"
#include "graphics.h"
#include "photo.h"

#pragma pack(1)
typedef struct
	{
	uint16_t		type ;				// Magic identifier
	uint32_t		size ;				// File size in bytes
	uint16_t		reserved1 ;			// Not used
	uint16_t		reserved2 ;			// Not used
	uint32_t		offset ;			// Offset to image data in bytes
	uint32_t		header_size ;		// Header size in bytes
	uint32_t		width ;				// Width of the image
	uint32_t		height ;			// Height of image
	uint16_t		planes ;			// Number of color planes
	uint16_t		bits ;				// Bits per pixel
	uint32_t		compression ;		// Compression type
	uint32_t		imagesize ;			// Image size in bytes
	uint32_t		xresolution ;		// Pixels per meter
	uint32_t		yresolution ;		// Pixels per meter
	uint32_t		ncolours ;			// Number of colors
	uint32_t		importantcolours ;	// Important colors
	} BMP_HEADER ;
#pragma pack()

typedef struct
	{
	uint8_t		blu ;	// 0 to 255
	uint8_t		grn ;	// 0 to 255
	uint8_t		red ;	// 0 to 255
	} PIXEL ;

void PaintImage(void *) ;

int main(void)
	{
	InitializeHardware(HEADER, "Photo") ;
	PaintImage(photo_bmp) ;
	while (1) ;
	}

void PaintImage(void *bmp)
	{
	int row, col, rows, cols, xoff, yoff, rlen ;
	BMP_HEADER *header = (BMP_HEADER *) bmp ;
    uint8_t *prow ;
	PIXEL *ppxl ;

	rows = header->height ;
	cols = header->width ;

	prow = (uint8_t *) (header + 1) ;
	rlen = (cols * sizeof(PIXEL) + 3) & ~0x3  ;

    xoff = (XPIXELS - cols) / 2 ;
    yoff = (YPIXELS - rows) / 2 + 15 ;

	for (row = 0; row < rows; row++)
		{
        ppxl = (PIXEL *) prow ;
		for (col = 0; col < cols; col++)
			{
            uint32_t rgb = *((uint32_t *) ppxl++) ;
			DrawPixel(xoff + col, yoff + rows - row, rgb | 0xFF000000) ;
			}
        prow += rlen ;
		}
	}

